//
//	(c) Paul Campbell 2012 - released under LGPL
//	This file is part of the DSPace Robot support software.
//
//	The DSPace Robot support software is free software: you can
// 	redistribute it and/or modify it under the terms of the
//	Lesser GNU General Public License as published by the Free
//	Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	The DSPace Robot support software is distributed in the hope
//	that it will be useful, but WITHOUT ANY WARRANTY; without even
//	the implied warranty of MERCHANTABILITY or FITNESS FOR A
//	PARTICULAR PURPOSE.  See the Lesser GNU General Public License
//	for more details.
//
//	You should have received a copy of the lesser GNU General Public
//	License along with the DSPace Robot support software.  If not,
//	see <http://www.gnu.org/licenses/>.
//
#ifndef _LINE_FOLLOWER_H_
#define _LINE_FOLLOWER_H_
#include "robot.h"

class LineFollowerRobotSensor: public I2CRobotSensor { // a sensor for the dspace 
public:
      static const int ADDRESS = 0x74;
      static const int   RED0   = 0x10;
      static const int   RED1   = 0x20;
      static const int   GREEN0 = 0x04;
      static const int   GREEN1 = 0x08;
      static const int   BLUE0  = 0x01;
      static const int   BLUE1  = 0x02;
      static const int SET_LEDS  = 0x00;
      static const int SCAN      = 0x40;
      static const int READ_0    = 0xc0;
      static const int READ_1    = 0xc1;
      static const int READ_01   = 0xc2;
      static const int SET_THRESH = 0xc3;
      static const int READ_THRESHOLD = 0xc4;
      
      LineFollowerRobotSensor(int n): I2CRobotSensor(n,1){}
      void begin(int xd=5) { I2CRobotSensor::begin(xd); WriteAddress(ADDRESS, SET_LEDS|0); }
      void Red() {  WriteAddress(ADDRESS, SET_LEDS|RED0|RED1); }
      void Green() {  WriteAddress(ADDRESS, SET_LEDS|GREEN0|GREEN1); }
      void Blue() {  WriteAddress(ADDRESS, SET_LEDS|BLUE0|BLUE1); }
      void Red(int n) {  WriteAddress(ADDRESS, SET_LEDS|(n?RED1:RED0)); }
      void Green(int n) {  WriteAddress(ADDRESS, SET_LEDS|(n?GREEN1:GREEN0)); }
      void Blue(int n) {  WriteAddress(ADDRESS, SET_LEDS|(n?BLUE1:BLUE0)); }
      void Off() {  WriteAddress(ADDRESS, SET_LEDS|0); }
      void Set(int r, int g, int b) {  WriteAddress(ADDRESS, SET_LEDS|(r?RED0|RED1:0)|(g?GREEN0|GREEN1:0)|(b?BLUE0|BLUE1:0)); }
      void Auto() { WriteAddress(ADDRESS, SCAN|RED0|RED1|GREEN0|GREEN1|BLUE0|BLUE1); }
      //void Auto() { WriteAddress(ADDRESS, SCAN|RED0|GREEN0|BLUE0); }
      void Auto(int n) { WriteAddress(ADDRESS, SCAN|(n&0x3f)); }
      void Auto(bool r, bool g, bool b) {  WriteAddress(ADDRESS, SCAN|(r?RED0|RED1:0)|(g?GREEN0|GREEN1:0)|(b?BLUE0|BLUE1:0)); }
      int Read0();
      int Read1();
      void ReadThreshold(int &r1, int &g1, int &b1, int &r2, int &g2, int &b2);
      void Read01(int &v1, int &v2);
      void SetThreshold(int r1, int g1, int b1, int r2, int g2, int b3);

};

#endif
