//
//	(c) Paul Campbell 2012 - released under LGPL
//	This file is part of the DSPace Robot support software.
//
//	The DSPace Robot support software is free software: you can
// 	redistribute it and/or modify it under the terms of the
//	Lesser GNU General Public License as published by the Free
//	Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	The DSPace Robot support software is distributed in the hope
//	that it will be useful, but WITHOUT ANY WARRANTY; without even
//	the implied warranty of MERCHANTABILITY or FITNESS FOR A
//	PARTICULAR PURPOSE.  See the Lesser GNU General Public License
//	for more details.
//
//	You should have received a copy of the lesser GNU General Public
//	License along with the DSPace Robot support software.  If not,
//	see <http://www.gnu.org/licenses/>.
//
#ifndef __SEEED__H
#define __SEEED__H

//
//	robot sensors for seed parts
//

#include "robot.h"


// Grove 

class GroveIRProximitySensor:public RobotSensor {
public:
	GroveIRProximitySensor(int s):RobotSensor(s) {}

	int Value() { return AnalogRead(); }
	int AverageValue(int n);
};



class GroveUltrasonicRangeSensor:public RobotSensor1Pin {
public:
	GroveUltrasonicRangeSensor(int s, SensorPin p=Pin0):RobotSensor1Pin(s, p) {PinMode(OUTPUT); DigitalWrite(LOW);}

	long Range() { Ping(); PinMode(OUTPUT); return PulseIn(HIGH);}	// time in microseconds
	long RangeCm() { return Range()/29/2;}
	
private:
	void Ping();
};

class GroveColourSensor:public I2CRobotSensor {
public:
	static const int ADDRESS = 0x39;
	static const int REG_CTL = 0x80;
	static const int REG_TIMING = 0x81;
		static const int SYNC_EDGE = 0x40;
		static const int INTEG_MODE_FREE = 0x00;
		static const int INTEG_MODE_MANUAL = 0x10;
		static const int INTEG_MODE_SYN_SINGLE = 0x20;
		static const int INTEG_MODE_SYN_MULTI = 0x30;
		static const int INTEG_PARAM_PULSE_COUNT1 = 0x00;
		static const int INTEG_PARAM_PULSE_COUNT2 = 0x01;
		static const int INTEG_PARAM_PULSE_COUNT4 = 0x02;
		static const int INTEG_PARAM_PULSE_COUNT8 = 0x03;
	static const int REG_INT = 0x82;
		static const int INTR_STOP = 40;
		static const int INTR_DISABLE = 0x00;
		static const int INTR_LEVEL = 0x10;
		static const int INTR_PERSIST_EVERY = 0x00;
		static const int INTR_PERSIST_SINGLE = 0x01;
	static const int REG_INT_SOURCE = 0x83;
		static const int INT_SOURCE_GREEN = 0x00;
		static const int INT_SOURCE_RED = 0x01;
		static const int INT_SOURCE_BLUE = 0x10;
		static const int INT_SOURCE_CLEAR = 0x03;
	static const int REG_ID = 0x84;
	static const int REG_GAIN = 0x87;
		static const int GAIN_1 = 0x00;
		static const int GAIN_4 = 0x10;
		static const int GAIN_16 = 0x20;
		static const int GANI_64 = 0x30;
		static const int PRESCALER_1 = 0x00;
		static const int PRESCALER_2 = 0x01;
		static const int PRESCALER_4 = 0x02;
		static const int PRESCALER_8 = 0x03;
		static const int PRESCALER_16 = 0x04;
		static const int PRESCALER_32 = 0x05;
		static const int PRESCALER_64 = 0x06;
	static const int REG_LOW_THRESH_LOW_BYTE = 0x88;
	static const int REG_LOW_THRESH_HIGH_BYTE = 0x89;
	static const int REG_HIGH_THRESH_LOW_BYTE = 0x8A;
	static const int REG_HIGH_THRESH_HIGH_BYTE = 0x8B;
	static const int REG_BLOCK_READ = 0xCF;
	static const int REG_GREEN_LOW = 0xD0;
	static const int REG_GREEN_HIGH = 0xD1;
	static const int REG_RED_LOW = 0xD2;
	static const int REG_RED_HIGH = 0xD3;
	static const int REG_BLUE_LOW = 0xD4;
	static const int REG_BLUE_HIGH = 0xD5;
	static const int REG_CLEAR_LOW = 0xD6;
	static const int REG_CLEAR_HIGH = 0xD7;
	static const int CTL_DAT_INIITIATE = 0x03;
	static const int CLR_INT = 0xE0;

	GroveColourSensor(int s): I2CRobotSensor(s) {}

	void Enable();
	void ReadRGB(int &red, int &green, int &blue, int &clear);
};
#endif
